@@ -224,6 +224,7 @@ config_print_zone(nsd_options_t* opt, const char* k, const char *o, const char *
 				ZONE_GET_ACL(allow_notify, o);
 				ZONE_GET_ACL(notify, o);
 				ZONE_GET_ACL(outgoing_interface, o);
+				ZONE_GET_BIN(allow_axfr_fallback, o);
 				printf("Zone option not handled: %s %s\n", z, o);
 				exit(1);
 			}
@@ -316,6 +317,7 @@ config_test_print_server(nsd_options_t* opt)
 		print_acl("notify:", zone->notify);
 		print_acl("provide-xfr:", zone->provide_xfr);
 		print_acl("outgoing-interface:", zone->outgoing_interface);
+		printf("\tallow-axfr-fallback: %s\n", zone->allow_axfr_fallback?"yes":"no");
 	}
 
 }
