@@ -30,6 +30,9 @@ usage(void)
 	fprintf(stderr, "-f		Force writing of zone files.\n");
 	fprintf(stderr, "-h		Print this help information.\n");
 	fprintf(stderr, "-l		List contents of transfer journal difffile, %s\n", DIFFFILE);
+	fprintf(stderr, "-o dbfile	Specify dbfile to output the result "
+			"directly to dbfile, nsd.db.\n");
+	fprintf(stderr, "-s		Skip writing of zone files.\n");
 	fprintf(stderr, "-x difffile	Specify diff file to use, instead of diff file from config.\n");
 	exit(1);
 }
@@ -39,9 +42,7 @@ list_xfr(FILE *in)
 {
 	uint32_t timestamp[2];
 	uint32_t skiplen, len, new_serial;
-/*	int i; */
 	char zone_name[3072];
-/*	uint8_t hex_data; */
 	uint16_t id;
 	uint32_t seq_nr, len2;
 
@@ -56,19 +57,10 @@ list_xfr(FILE *in)
 		return;
 	}
 	skiplen = len - (sizeof(uint32_t)*3 + sizeof(uint16_t) + strlen(zone_name));
-	fprintf(stderr, "zone %s transfer id %x serial %d timestamp %u.%u: "
+	fprintf(stdout, "zone %s transfer id %x serial %d timestamp %u.%u: "
 			"seq_nr %d of %d bytes\n", zone_name, id, new_serial,
 		timestamp[0], timestamp[1], seq_nr, skiplen);
 
-/* Debug code, print the hexadecimal contents of the packet
-	needed for version 3.1.1
-	for (i=0; i<skiplen; i++) {
-			fread(&hex_data, 1, 1, in);
-			fprintf(stderr, " %2.2x ", hex_data);
-	}
-	fprintf(stderr, " \n");
-*/
-
 	if(fseeko(in, skiplen, SEEK_CUR) == -1)
 		fprintf(stderr, "fseek failed: %s\n", strerror(errno));
 
@@ -77,7 +69,7 @@ list_xfr(FILE *in)
 		return;
 	}
 	if(len != len2) {
-		fprintf(stderr, "Packet seq %d had bad length check bytes!\n",
+		fprintf(stderr, "packet seq %d had bad length check bytes!\n",
 			seq_nr);
 	}
 }
@@ -126,9 +118,9 @@ list_commit(FILE *in)
 		fprintf(stderr, "incomplete commit/rollback packet\n");
 		return;
 	}
-	fprintf(stderr, "zone %s transfer id %x serial %d: %s of %d packets\n",
+	fprintf(stdout, "zone %s transfer id %x serial %d: %s of %d packets\n",
 		zone_name, id, new_serial, commit?"commit":"rollback", num);
-	fprintf(stderr, "   time %s, from serial %d, log message: %s\n",
+	fprintf(stdout, "   time %s, from serial %d, log message: %s\n",
 		get_date(log_msg), old_serial, log_msg);
 	if(len != len2) {
 		fprintf(stderr, "  commit packet with bad length check \
@@ -142,10 +134,10 @@ debug_list(struct nsd_options* opt)
 	const char* file = opt->difffile;
 	FILE *f;
 	uint32_t type;
-	fprintf(stderr, "Debug listing of the contents of %s\n", file);
+	fprintf(stdout, "debug listing of the contents of %s\n", file);
 	f = fopen(file, "r");
 	if(!f) {
-		fprintf(stderr, "Error opening %s: %s\n", file,
+		fprintf(stderr, "error opening %s: %s\n", file,
 			strerror(errno));
 		return;
 	}
@@ -282,16 +274,20 @@ int main(int argc, char* argv[])
 	int c;
 	const char* configfile = CONFIGFILE;
 	const char* difffile = NULL;
+	const char* dbfile = NULL;
 	nsd_options_t *options;
-	struct namedb* db;
+	struct namedb* db = NULL;
+	struct namedb* dbout = NULL;
 	struct zone* zone;
 	struct diff_log* commit_log = 0;
 	size_t fake_child_count = 1;
 	int debug_list_diff = 0;
 	int force_write = 0;
+	int skip_write = 0;
+	int difffile_exists = 0;
 
         /* Parse the command line... */
-	while ((c = getopt(argc, argv, "c:fhlx:")) != -1) {
+	while ((c = getopt(argc, argv, "c:fhlo:sx:")) != -1) {
 	switch (c) {
 		case 'c':
 			configfile = optarg;
@@ -300,7 +296,27 @@ int main(int argc, char* argv[])
 			debug_list_diff = 1;
 			break;
 		case 'f':
-			force_write = 1;
+			if (skip_write)
+			{
+				fprintf(stderr, "Cannot force and skip writing "
+						"zonefiles at the same time\n");
+				exit(1);
+			}
+			else
+				force_write = 1;
+			break;
+		case 's':
+			if (force_write)
+			{
+				fprintf(stderr, "Cannot skip and force writing "
+						"zonefiles at the same time\n");
+				exit(1);
+			}
+			else
+				skip_write = 1;
+			break;
+		case 'o':
+			dbfile = optarg;
 			break;
 		case 'x':
 			difffile = optarg;
@@ -336,9 +352,11 @@ int main(int argc, char* argv[])
 
 	/* see if necessary */
 	if(!exist_difffile(options)) {
-		fprintf(stderr, "No diff file, nothing to do.\n");
-		exit(0);
+		fprintf(stderr, "No diff file.\n");
+		if (!force_write)
+			exit(0);
 	}
+	else	difffile_exists = 1;
 
 	if(debug_list_diff) {
 		debug_list(options);
@@ -346,10 +364,10 @@ int main(int argc, char* argv[])
 	}
 
 	/* read database and diff file */
-	printf("reading database\n");
+	fprintf(stdout, "reading database\n");
 	db = namedb_open(options->database, options, fake_child_count);
 	if(!db) {
-		fprintf(stderr, "Could not read database: %s\n",
+		fprintf(stderr, "could not read database: %s\n",
 			options->database);
 		exit(1);
 	}
@@ -360,26 +378,52 @@ int main(int argc, char* argv[])
 		zone->updated = 0;
 	}
 
-	/* read ixfr diff file */
-	fprintf(stderr, "reading updates to database\n");
-	if(!diff_read_file(db, options, &commit_log, fake_child_count)) {
-		fprintf(stderr, "unable to load the diff file: %s\n",
-			options->difffile);
-		exit(1);
+	if (dbfile)
+		dbout = namedb_new(dbfile);
+	if (dbout)
+	{
+		db->fd = dbout->fd;
+		db->filename = (char*) dbfile;
 	}
 
-	fprintf(stderr, "writing changed zones\n");
-	for(zone = db->zones; zone; zone = zone->next)
-	{
-		if(!force_write && !zone->updated) {
-			fprintf(stderr, "zone %s had not changed.\n",
-				zone->opts->name);
-			continue;
+	/* read ixfr diff file */
+	if (difffile_exists) {
+		fprintf(stdout, "reading updates to database\n");
+		if(!diff_read_file(db, options, &commit_log, fake_child_count))
+		{
+			fprintf(stderr, "unable to load the diff file: %s\n",
+				options->difffile);
+			exit(1);
 		}
-		/* write zone to its zone file */
-		write_to_zonefile(zone, commit_log);
 	}
-	fprintf(stderr, "done\n");
+
+	if (skip_write)
+		fprintf(stderr, "skip patching up zonefiles.\n");
+	else {
+		fprintf(stdout, "writing changed zones\n");
+		for(zone = db->zones; zone; zone = zone->next)
+		{
+			if(!force_write && !zone->updated) {
+				fprintf(stdout, "zone %s had not changed.\n",
+					zone->opts->name);
+				continue;
+			}
+			/* write zone to its zone file */
+			write_to_zonefile(zone, commit_log);
+		}
+	}
+
+	/* output result directly to dbfile */
+	if (dbout)
+	{
+		fprintf(stdout, "storing database to %s.\n", dbout->filename);
+	        if (namedb_save(db) != 0) {
+			fprintf(stderr, "error writing the database (%s): %s\n",
+				dbfile, strerror(errno));
+			exit(1);
+		}
+	}
+	fprintf(stdout, "done\n");
 
 	return 0;
 }
